==> qt-async-threads-0.3.1.tar.gz: qt-async-threads-0.3.1/docs/examples/explanation_async_parallel.py <==
from pathlib import Path
from urllib.parse import urlsplit

import requests
from examples.exception import install_except_hook
from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QFormLayout
from PyQt5.QtWidgets import QLabel
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtWidgets import QSpinBox
from PyQt5.QtWidgets import QWidget
from qt_async_threads import QtAsyncRunner
from requests import Response
from requests.exceptions import ConnectionError


class Window(QWidget):
    def __init__(self, directory: Path, runner: QtAsyncRunner) -> None:
        super().__init__()
        self.runner = runner

        self.setWindowTitle("Cat Downloader")
        self.directory = directory
        self._cancelled = False

        # Build controls.
        self.count_spin = QSpinBox()
        self.count_spin.setValue(5)
        self.count_spin.setMinimum(1)
        self.progress_label = QLabel("Idle, click below to start downloading")
        self.download_button = QPushButton("Download")
        self.stop_button = QPushButton("Stop")
        self.stop_button.setEnabled(False)

        layout = QFormLayout(self)
        layout.addRow("How many cats?", self.count_spin)
        layout.addRow("Status", self.progress_label)
        layout.addRow(self.download_button)
        layout.addRow(self.stop_button)

        # Connect signals.
        self.download_button.clicked.connect(self.runner.to_sync(self.on_download_button_clicked))
        self.stop_button.clicked.connect(self.on_cancel_button_clicked)

    async def on_download_button_clicked(self, checked: bool = False) -> None:
        self.progress_label.setText("Searching...")
        self.download_button.setEnabled(False)
        self.stop_button.setEnabled(True)

        self._cancelled = False
        downloaded_count = 0

        def download_one() -> Response:
            # Search.
            search_response = requests.get("https://api.thecatapi.com/v1/images/search")
            search_response.raise_for_status()

            # Download.
            url = search_response.json()[0]["url"]
            return requests.get(url)

        try:
            functions = [download_one for _ in range(self.count_spin.value())]
            async for download_response in self.runner.run_parallel(functions):
                # Save the contents of the image to a file.
                parts = urlsplit(download_response.url)
                path = self.directory / f"{downloaded_count:02d}_cat{Path(parts.path).suffix}"
                path.write_bytes(download_response.content)
                downloaded_count += 1

                # Show progress.
                self.progress_label.setText(f"Downloaded {path.name}")
                QApplication.processEvents()
                if self._cancelled:
                    QMessageBox.information(self, "Cancelled", "Download cancelled")
                    return
        except ConnectionError as e:
            QMessageBox.critical(self, "Error", f"Error connecting to TheCatApi:\n{e}")
            return
        finally:
            self.progress_label.setText(f"Done, {downloaded_count} cats downloaded")
            self.download_button.setEnabled(True)
            self.stop_button.setEnabled(False)

    def on_cancel_button_clicked(self) -> None:
        self._cancelled = True


if __name__ == "__main__":
    install_except_hook()
    with QtAsyncRunner() as runner:
        app = QApplication([])
        win = Window(Path(__file__).parent / "cats", runner)
        win.show()
        app.exec()

==> qt-async-threads-0.4.0.tar.gz: qt-async-threads-0.4.0/docs/examples/explanation_async_parallel.py <==
from pathlib import Path
from urllib.parse import urlsplit

import requests
from examples.exception import install_except_hook
from qt_async_threads import QtAsyncRunner
from qtpy.QtWidgets import QApplication
from qtpy.QtWidgets import QFormLayout
from qtpy.QtWidgets import QLabel
from qtpy.QtWidgets import QMessageBox
from qtpy.QtWidgets import QPushButton
from qtpy.QtWidgets import QSpinBox
from qtpy.QtWidgets import QWidget
from requests import Response
from requests.exceptions import ConnectionError


class Window(QWidget):
    def __init__(self, directory: Path, runner: QtAsyncRunner) -> None:
        super().__init__()
        self.runner = runner

        self.setWindowTitle("Cat Downloader")
        self.directory = directory
        self._cancelled = False

        # Build controls.
        self.count_spin = QSpinBox()
        self.count_spin.setValue(5)
        self.count_spin.setMinimum(1)
        self.progress_label = QLabel("Idle, click below to start downloading")
        self.download_button = QPushButton("Download")
        self.stop_button = QPushButton("Stop")
        self.stop_button.setEnabled(False)

        layout = QFormLayout(self)
        layout.addRow("How many cats?", self.count_spin)
        layout.addRow("Status", self.progress_label)
        layout.addRow(self.download_button)
        layout.addRow(self.stop_button)

        # Connect signals.
        self.download_button.clicked.connect(self.runner.to_sync(self.on_download_button_clicked))
        self.stop_button.clicked.connect(self.on_cancel_button_clicked)

    async def on_download_button_clicked(self, checked: bool = False) -> None:
        self.progress_label.setText("Searching...")
        self.download_button.setEnabled(False)
        self.stop_button.setEnabled(True)

        self._cancelled = False
        downloaded_count = 0

        def download_one() -> Response:
            # Search.
            search_response = requests.get("https://api.thecatapi.com/v1/images/search")
            search_response.raise_for_status()

            # Download.
            url = search_response.json()[0]["url"]
            return requests.get(url)

        try:
            functions = [download_one for _ in range(self.count_spin.value())]
            async for download_response in self.runner.run_parallel(functions):
                # Save the contents of the image to a file.
                parts = urlsplit(download_response.url)
                path = self.directory / f"{downloaded_count:02d}_cat{Path(parts.path).suffix}"
                path.write_bytes(download_response.content)
                downloaded_count += 1

                # Show progress.
                self.progress_label.setText(f"Downloaded {path.name}")
                QApplication.processEvents()
                if self._cancelled:
                    QMessageBox.information(self, "Cancelled", "Download cancelled")
                    return
        except ConnectionError as e:
            QMessageBox.critical(self, "Error", f"Error connecting to TheCatApi:\n{e}")
            return
        finally:
            self.progress_label.setText(f"Done, {downloaded_count} cats downloaded")
            self.download_button.setEnabled(True)
            self.stop_button.setEnabled(False)

    def on_cancel_button_clicked(self) -> None:
        self._cancelled = True


if __name__ == "__main__":
    install_except_hook()
    with QtAsyncRunner() as runner:
        app = QApplication([])
        win = Window(Path(__file__).parent / "cats", runner)
        win.show()
        app.exec()
